@@ -14,6 +14,5 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from importlib import metadata
 
-__version__ = metadata.version(__package__)
+__version__ = "0.6.0"
