@@ -16,62 +16,65 @@
 # under the License.
 import json
 from abc import ABC, abstractmethod
-from typing import Any, List, Optional
+from typing import (
+    Any,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+)
 from uuid import UUID
 
 from rich.console import Console
 from rich.table import Table as RichTable
 from rich.tree import Tree
 
-from pyiceberg.io import FileIO
+from pyiceberg.partitioning import PartitionSpec
 from pyiceberg.schema import Schema
-from pyiceberg.table import Table
-from pyiceberg.table.partitioning import PartitionSpec
-from pyiceberg.typedef import Identifier, Properties
+from pyiceberg.table import Table, TableMetadata
+from pyiceberg.table.refs import SnapshotRefType
+from pyiceberg.typedef import IcebergBaseModel, Identifier, Properties
 
 
 class Output(ABC):
-    """Output interface for exporting"""
+    """Output interface for exporting."""
 
     @abstractmethod
-    def exception(self, ex: Exception) -> None:
-        ...
+    def exception(self, ex: Exception) -> None: ...
 
     @abstractmethod
-    def identifiers(self, identifiers: List[Identifier]) -> None:
-        ...
+    def identifiers(self, identifiers: List[Identifier]) -> None: ...
 
     @abstractmethod
-    def describe_table(self, table: Table) -> None:
-        ...
+    def describe_table(self, table: Table) -> None: ...
 
     @abstractmethod
-    def files(self, table: Table, io: FileIO, history: bool) -> None:
-        ...
+    def files(self, table: Table, history: bool) -> None: ...
 
     @abstractmethod
-    def describe_properties(self, properties: Properties) -> None:
-        ...
+    def describe_properties(self, properties: Properties) -> None: ...
 
     @abstractmethod
-    def text(self, response: str) -> None:
-        ...
+    def text(self, response: str) -> None: ...
 
     @abstractmethod
-    def schema(self, schema: Schema) -> None:
-        ...
+    def schema(self, schema: Schema) -> None: ...
 
     @abstractmethod
-    def spec(self, spec: PartitionSpec) -> None:
-        ...
+    def spec(self, spec: PartitionSpec) -> None: ...
 
     @abstractmethod
-    def uuid(self, uuid: Optional[UUID]) -> None:
-        ...
+    def uuid(self, uuid: Optional[UUID]) -> None: ...
+
+    @abstractmethod
+    def version(self, version: str) -> None: ...
+
+    @abstractmethod
+    def describe_refs(self, refs: List[Tuple[str, SnapshotRefType, Dict[str, str]]]) -> None: ...
 
 
 class ConsoleOutput(Output):
-    """Writes to the console"""
+    """Writes to the console."""
 
     verbose: bool
 
@@ -124,7 +127,7 @@ class ConsoleOutput(Output):
         output_table.add_row("Properties", table_properties)
         Console().print(output_table)
 
-    def files(self, table: Table, io: FileIO, history: bool) -> None:
+    def files(self, table: Table, history: bool) -> None:
         if history:
             snapshots = table.metadata.snapshots
         else:
@@ -134,15 +137,16 @@ class ConsoleOutput(Output):
                 snapshots = []
 
         snapshot_tree = Tree(f"Snapshots: {'.'.join(table.identifier)}")
+        io = table.io
 
         for snapshot in snapshots:
             manifest_list_str = f": {snapshot.manifest_list}" if snapshot.manifest_list else ""
             list_tree = snapshot_tree.add(f"Snapshot {snapshot.snapshot_id}, schema {snapshot.schema_id}{manifest_list_str}")
 
-            manifest_list = snapshot.fetch_manifest_list(io)
+            manifest_list = snapshot.manifests(io)
             for manifest in manifest_list:
                 manifest_tree = list_tree.add(f"Manifest: {manifest.manifest_path}")
-                for manifest_entry in manifest.fetch_manifest_entry(io):
+                for manifest_entry in manifest.fetch_manifest_entry(io, discard_deleted=False):
                     manifest_tree.add(f"Datafile: {manifest_entry.data_file.file_path}")
         Console().print(snapshot_tree)
 
@@ -167,9 +171,25 @@ class ConsoleOutput(Output):
     def uuid(self, uuid: Optional[UUID]) -> None:
         Console().print(str(uuid) if uuid else "missing")
 
+    def version(self, version: str) -> None:
+        Console().print(version)
+
+    def describe_refs(self, ref_details: List[Tuple[str, SnapshotRefType, Dict[str, str]]]) -> None:
+        refs_table = RichTable(title="Snapshot Refs")
+        refs_table.add_column("Ref")
+        refs_table.add_column("Type")
+        refs_table.add_column("Max ref age ms")
+        refs_table.add_column("Min snapshots to keep")
+        refs_table.add_column("Max snapshot age ms")
+        for name, type, ref_detail in ref_details:
+            refs_table.add_row(
+                name, type, ref_detail["max_ref_age_ms"], ref_detail["min_snapshots_to_keep"], ref_detail["max_snapshot_age_ms"]
+            )
+        Console().print(refs_table)
+
 
 class JsonOutput(Output):
-    """Writes json to stdout"""
+    """Writes json to stdout."""
 
     verbose: bool
 
@@ -186,7 +206,18 @@ class JsonOutput(Output):
         self._out([".".join(identifier) for identifier in identifiers])
 
     def describe_table(self, table: Table) -> None:
-        print(table.json())
+        class FauxTable(IcebergBaseModel):
+            """Just to encode it using Pydantic."""
+
+            identifier: Identifier
+            metadata_location: str
+            metadata: TableMetadata
+
+        print(
+            FauxTable(
+                identifier=table.identifier, metadata=table.metadata, metadata_location=table.metadata_location
+            ).model_dump_json()
+        )
 
     def describe_properties(self, properties: Properties) -> None:
         self._out(properties)
@@ -195,13 +226,23 @@ class JsonOutput(Output):
         print(json.dumps(response))
 
     def schema(self, schema: Schema) -> None:
-        print(schema.json())
+        print(schema.model_dump_json())
 
-    def files(self, table: Table, io: FileIO, history: bool) -> None:
+    def files(self, table: Table, history: bool) -> None:
         pass
 
     def spec(self, spec: PartitionSpec) -> None:
-        print(spec.json())
+        print(spec.model_dump_json())
 
     def uuid(self, uuid: Optional[UUID]) -> None:
         self._out({"uuid": str(uuid) if uuid else "missing"})
+
+    def version(self, version: str) -> None:
+        self._out({"version": version})
+
+    def describe_refs(self, refs: List[Tuple[str, SnapshotRefType, Dict[str, str]]]) -> None:
+        self._out([
+            {"name": name, "type": type, detail_key: detail_val}
+            for name, type, detail in refs
+            for detail_key, detail_val in detail.items()
+        ])
