@@ -14,13 +14,19 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-"""Utility class for converting between Avro and Iceberg schemas"""
-from __future__ import annotations
+"""Utility class for converting between Avro and Iceberg schemas."""
 
 import logging
-from typing import Any
+from typing import (
+    Any,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Union,
+)
 
-from pyiceberg.schema import Schema
+from pyiceberg.schema import Schema, SchemaVisitorPerPrimitiveType, visit
 from pyiceberg.types import (
     BinaryType,
     BooleanType,
@@ -39,13 +45,15 @@ from pyiceberg.types import (
     StringType,
     StructType,
     TimestampType,
+    TimestamptzType,
     TimeType,
     UUIDType,
 )
+from pyiceberg.utils.decimal import decimal_required_bytes
 
 logger = logging.getLogger(__name__)
 
-PRIMITIVE_FIELD_TYPE_MAPPING: dict[str, PrimitiveType] = {
+PRIMITIVE_FIELD_TYPE_MAPPING: Dict[str, PrimitiveType] = {
     "boolean": BooleanType(),
     "bytes": BinaryType(),
     "double": DoubleType(),
@@ -56,21 +64,21 @@ PRIMITIVE_FIELD_TYPE_MAPPING: dict[str, PrimitiveType] = {
     "enum": StringType(),
 }
 
-LOGICAL_FIELD_TYPE_MAPPING: dict[tuple[str, str], PrimitiveType] = {
+LOGICAL_FIELD_TYPE_MAPPING: Dict[Tuple[str, str], PrimitiveType] = {
     ("date", "int"): DateType(),
-    ("time-millis", "int"): TimeType(),
-    ("timestamp-millis", "long"): TimestampType(),
-    ("time-micros", "int"): TimeType(),
+    ("time-micros", "long"): TimeType(),
     ("timestamp-micros", "long"): TimestampType(),
-    ("uuid", "string"): UUIDType(),
+    ("uuid", "fixed"): UUIDType(),
 }
 
+AvroType = Union[str, Any]
+
 
 class AvroSchemaConversion:
-    def avro_to_iceberg(self, avro_schema: dict[str, Any]) -> Schema:
-        """Converts an Apache Avro into an Apache Iceberg schema equivalent
+    def avro_to_iceberg(self, avro_schema: Dict[str, Any]) -> Schema:
+        """Convert an Apache Avro into an Apache Iceberg schema equivalent.
 
-        This expects to have field id's to be encoded in the Avro schema::
+        This expects to have field id's to be encoded in the Avro schema:
 
             {
                 "type": "record",
@@ -105,16 +113,22 @@ class AvroSchemaConversion:
             True
 
         Args:
-            avro_schema (Dict[str, Any]): The JSON decoded Avro schema
+            avro_schema (Dict[str, Any]): The JSON decoded Avro schema.
 
         Returns:
-            Equivalent Iceberg schema
+            Equivalent Iceberg schema.
         """
         return Schema(*[self._convert_field(field) for field in avro_schema["fields"]], schema_id=1)
 
-    def _resolve_union(self, type_union: dict | list | str) -> tuple[str | dict[str, Any], bool]:
+    def iceberg_to_avro(self, schema: Schema, schema_name: Optional[str] = None) -> AvroType:
+        """Convert an Iceberg schema into an Avro dictionary that can be serialized to JSON."""
+        return visit(schema, ConvertSchemaToAvro(schema_name))
+
+    def _resolve_union(
+        self, type_union: Union[Dict[str, str], List[Union[str, Dict[str, str]]], str]
+    ) -> Tuple[Union[str, Dict[str, Any]], bool]:
         """
-        Converts Unions into their type and resolves if the field is required
+        Convert Unions into their type and resolves if the field is required.
 
         Examples:
             >>> AvroSchemaConversion()._resolve_union('str')
@@ -127,15 +141,15 @@ class AvroSchemaConversion:
             ({'type': 'str'}, False)
 
         Args:
-            type_union: The field, can be a string 'str', list ['null', 'str'], or dict {"type": 'str'}
+            type_union: The field, can be a string 'str', list ['null', 'str'], or dict {"type": 'str'}.
 
         Returns:
-            A tuple containing the type and if required
+            A tuple containing the type and if required.
 
         Raises:
-            TypeError: In the case non-optional union types are encountered
+            TypeError: In the case non-optional union types are encountered.
         """
-        avro_types: dict | list
+        avro_types: Union[Dict[str, str], List[Union[Dict[str, str], str]]]
         if isinstance(type_union, str):
             # It is a primitive and required
             return type_union, True
@@ -161,15 +175,15 @@ class AvroSchemaConversion:
         # Filter the null value and return the type
         return list(filter(lambda t: t != "null", avro_types))[0], False
 
-    def _convert_schema(self, avro_type: str | dict[str, Any]) -> IcebergType:
+    def _convert_schema(self, avro_type: Union[str, Dict[str, Any]]) -> IcebergType:
         """
-        Resolves the Avro type
+        Resolve the Avro type.
 
         Args:
-            avro_type: The Avro type, can be simple or complex
+            avro_type: The Avro type, can be simple or complex.
 
         Returns:
-            The equivalent IcebergType
+            The equivalent IcebergType.
 
         Raises:
             ValueError: When there are unknown types
@@ -199,14 +213,14 @@ class AvroSchemaConversion:
         else:
             raise TypeError(f"Unknown type: {avro_type}")
 
-    def _convert_field(self, field: dict[str, Any]) -> NestedField:
-        """
-        Converts an Avro field into an Iceberg equivalent field
+    def _convert_field(self, field: Dict[str, Any]) -> NestedField:
+        """Convert an Avro field into an Iceberg equivalent field.
+
         Args:
-            field: The Avro field
+            field: The Avro field.
 
         Returns:
-            The Iceberg equivalent field
+            The Iceberg equivalent field.
         """
         if "field-id" not in field:
             raise ValueError(f"Cannot convert field, missing field-id: {field}")
@@ -221,9 +235,9 @@ class AvroSchemaConversion:
             doc=field.get("doc"),
         )
 
-    def _convert_record_type(self, record_type: dict[str, Any]) -> StructType:
+    def _convert_record_type(self, record_type: Dict[str, Any]) -> StructType:
         """
-        Converts the fields from a record into an Iceberg struct
+        Convert the fields from a record into an Iceberg struct.
 
         Examples:
             >>> from pyiceberg.utils.schema_conversion import AvroSchemaConversion
@@ -266,16 +280,16 @@ class AvroSchemaConversion:
             True
 
         Args:
-            record_type: The record type itself
+            record_type: The record type itself.
 
-        Returns:
+        Returns: A StructType.
         """
         if record_type["type"] != "record":
             raise ValueError(f"Expected record type, got: {record_type}")
 
         return StructType(*[self._convert_field(field) for field in record_type["fields"]])
 
-    def _convert_array_type(self, array_type: dict[str, Any]) -> ListType:
+    def _convert_array_type(self, array_type: Dict[str, Any]) -> ListType:
         if "element-id" not in array_type:
             raise ValueError(f"Cannot convert array-type, missing element-id: {array_type}")
 
@@ -287,10 +301,11 @@ class AvroSchemaConversion:
             element_required=element_required,
         )
 
-    def _convert_map_type(self, map_type: dict[str, Any]) -> MapType:
-        """
+    def _convert_map_type(self, map_type: Dict[str, Any]) -> MapType:
+        """Convert an avro map type into an Iceberg MapType.
+
         Args:
-            map_type: The dict that describes the Avro map type
+            map_type: The dict that describes the Avro map type.
 
         Examples:
             >>> from pyiceberg.utils.schema_conversion import AvroSchemaConversion
@@ -311,7 +326,7 @@ class AvroSchemaConversion:
             >>> actual == expected
             True
 
-        Returns: A MapType
+        Returns: A MapType.
         """
         value_type, value_required = self._resolve_union(map_type["values"])
         return MapType(
@@ -323,11 +338,11 @@ class AvroSchemaConversion:
             value_required=value_required,
         )
 
-    def _convert_logical_type(self, avro_logical_type: dict[str, Any]) -> IcebergType:
-        """
-        Convert a schema with a logical type annotation. For the decimal and map
-        we need to fetch more keys from the dict, and for the simple ones we can just
-        look it up in the mapping.
+    def _convert_logical_type(self, avro_logical_type: Dict[str, Any]) -> IcebergType:
+        """Convert a schema with a logical type annotation into an IcebergType.
+
+        For the decimal and map we need to fetch more keys from the dict, and for
+        the simple ones we can just look it up in the mapping.
 
         Examples:
             >>> from pyiceberg.utils.schema_conversion import AvroSchemaConversion
@@ -340,13 +355,13 @@ class AvroSchemaConversion:
             True
 
         Args:
-            avro_logical_type: The logical type
+            avro_logical_type: The logical type.
 
         Returns:
-            The converted logical type
+            The converted logical type.
 
         Raises:
-            ValueError: When the logical type is unknown
+            ValueError: When the logical type is unknown.
         """
         logical_type = avro_logical_type["logicalType"]
         physical_type = avro_logical_type["type"]
@@ -354,15 +369,21 @@ class AvroSchemaConversion:
             return self._convert_logical_decimal_type(avro_logical_type)
         elif logical_type == "map":
             return self._convert_logical_map_type(avro_logical_type)
+        elif logical_type == "timestamp-micros":
+            if avro_logical_type.get("adjust-to-utc", False) is True:
+                return TimestamptzType()
+            else:
+                return TimestampType()
         elif (logical_type, physical_type) in LOGICAL_FIELD_TYPE_MAPPING:
             return LOGICAL_FIELD_TYPE_MAPPING[(logical_type, physical_type)]
         else:
             raise ValueError(f"Unknown logical/physical type combination: {avro_logical_type}")
 
-    def _convert_logical_decimal_type(self, avro_type: dict[str, Any]) -> DecimalType:
-        """
+    def _convert_logical_decimal_type(self, avro_type: Dict[str, Any]) -> DecimalType:
+        """Convert an avro type to an Iceberg DecimalType.
+
         Args:
-            avro_type: The Avro type
+            avro_type: The Avro type.
 
         Examples:
             >>> from pyiceberg.utils.schema_conversion import AvroSchemaConversion
@@ -381,17 +402,17 @@ class AvroSchemaConversion:
             True
 
         Returns:
-            A Iceberg DecimalType
+            A Iceberg DecimalType.
         """
         return DecimalType(precision=avro_type["precision"], scale=avro_type["scale"])
 
-    def _convert_logical_map_type(self, avro_type: dict[str, Any]) -> MapType:
-        """
-        In the case where a map hasn't a key as a type you can use a logical map to
-        still encode this in Avro
+    def _convert_logical_map_type(self, avro_type: Dict[str, Any]) -> MapType:
+        """Convert an avro map type to an Iceberg MapType.
+
+        In the case where a map hasn't a key as a type you can use a logical map to still encode this in Avro.
 
         Args:
-            avro_type: The Avro Type
+            avro_type: The Avro Type.
 
         Examples:
             >>> from pyiceberg.utils.schema_conversion import AvroSchemaConversion
@@ -422,7 +443,7 @@ class AvroSchemaConversion:
             https://iceberg.apache.org/spec/#appendix-a-format-specific-requirements
 
         Returns:
-            The logical map
+            The logical map.
         """
         fields = avro_type["items"]["fields"]
         if len(fields) != 2:
@@ -437,12 +458,14 @@ class AvroSchemaConversion:
             value_required=value.required,
         )
 
-    def _convert_fixed_type(self, avro_type: dict[str, Any]) -> FixedType:
+    def _convert_fixed_type(self, avro_type: Dict[str, Any]) -> FixedType:
         """
-        https://avro.apache.org/docs/current/spec.html#Fixed
+        Convert Avro Type to the equivalent Iceberg fixed type.
+
+        - https://avro.apache.org/docs/current/spec.html#Fixed
 
         Args:
-            avro_type: The Avro Type
+            avro_type: The Avro type.
 
         Examples:
             >>> from pyiceberg.utils.schema_conversion import AvroSchemaConversion
@@ -455,6 +478,141 @@ class AvroSchemaConversion:
             True
 
         Returns:
-            An Iceberg equivalent fixed type
+            An Iceberg equivalent fixed type.
         """
         return FixedType(length=avro_type["size"])
+
+
+class ConvertSchemaToAvro(SchemaVisitorPerPrimitiveType[AvroType]):
+    """Convert an Iceberg schema to an Avro schema."""
+
+    schema_name: Optional[str]
+    last_list_field_id: int
+    last_map_key_field_id: int
+    last_map_value_field_id: int
+
+    def __init__(self, schema_name: Optional[str]) -> None:
+        """Convert an Iceberg schema to an Avro schema.
+
+        Args:
+            schema_name: The name of the root record.
+        """
+        self.schema_name = schema_name
+
+    def schema(self, schema: Schema, struct_result: AvroType) -> AvroType:
+        if isinstance(struct_result, dict) and self.schema_name is not None:
+            struct_result["name"] = self.schema_name
+        return struct_result
+
+    def before_list_element(self, element: NestedField) -> None:
+        self.last_list_field_id = element.field_id
+
+    def before_map_key(self, key: NestedField) -> None:
+        self.last_map_key_field_id = key.field_id
+
+    def before_map_value(self, value: NestedField) -> None:
+        self.last_map_value_field_id = value.field_id
+
+    def struct(self, struct: StructType, field_results: List[AvroType]) -> AvroType:
+        return {"type": "record", "fields": field_results}
+
+    def field(self, field: NestedField, field_result: AvroType) -> AvroType:
+        # Sets the schema name
+        if isinstance(field_result, dict) and field_result.get("type") == "record":
+            field_result["name"] = f"r{field.field_id}"
+
+        result = {
+            "name": field.name,
+            "field-id": field.field_id,
+            "type": field_result if field.required else ["null", field_result],
+        }
+
+        if field.optional:
+            result["default"] = None
+
+        if field.doc is not None:
+            result["doc"] = field.doc
+
+        return result
+
+    def list(self, list_type: ListType, element_result: AvroType) -> AvroType:
+        # Sets the schema name in case of a record
+        if isinstance(element_result, dict) and element_result.get("type") == "record":
+            element_result["name"] = f"r{self.last_list_field_id}"
+        return {"type": "array", "element-id": self.last_list_field_id, "items": element_result}
+
+    def map(self, map_type: MapType, key_result: AvroType, value_result: AvroType) -> AvroType:
+        if isinstance(key_result, StringType):
+            # Avro Maps does not support other keys than a String,
+            return {
+                "type": "map",
+                "values": value_result,
+                "key-id": self.last_map_key_field_id,
+                "value-id": self.last_map_value_field_id,
+            }
+        else:
+            # Creates a logical map that's a list of schema's
+            # binary compatible
+            return {
+                "type": "array",
+                "items": {
+                    "type": "record",
+                    "name": f"k{self.last_map_key_field_id}_v{self.last_map_value_field_id}",
+                    "fields": [
+                        {"name": "key", "type": key_result, "field-id": self.last_map_key_field_id},
+                        {"name": "value", "type": value_result, "field-id": self.last_map_value_field_id},
+                    ],
+                },
+                "logicalType": "map",
+            }
+
+    def visit_fixed(self, fixed_type: FixedType) -> AvroType:
+        return {"type": "fixed", "size": len(fixed_type), "name": f"fixed_{len(fixed_type)}"}
+
+    def visit_decimal(self, decimal_type: DecimalType) -> AvroType:
+        return {
+            "type": "fixed",
+            "size": decimal_required_bytes(decimal_type.precision),
+            "logicalType": "decimal",
+            "precision": decimal_type.precision,
+            "scale": decimal_type.scale,
+            "name": f"decimal_{decimal_type.precision}_{decimal_type.scale}",
+        }
+
+    def visit_boolean(self, boolean_type: BooleanType) -> AvroType:
+        return "boolean"
+
+    def visit_integer(self, integer_type: IntegerType) -> AvroType:
+        return "int"
+
+    def visit_long(self, long_type: LongType) -> AvroType:
+        return "long"
+
+    def visit_float(self, float_type: FloatType) -> AvroType:
+        return "float"
+
+    def visit_double(self, double_type: DoubleType) -> AvroType:
+        return "double"
+
+    def visit_date(self, date_type: DateType) -> AvroType:
+        return {"type": "int", "logicalType": "date"}
+
+    def visit_time(self, time_type: TimeType) -> AvroType:
+        return {"type": "long", "logicalType": "time-micros"}
+
+    def visit_timestamp(self, timestamp_type: TimestampType) -> AvroType:
+        # Iceberg only supports micro's
+        return {"type": "long", "logicalType": "timestamp-micros", "adjust-to-utc": False}
+
+    def visit_timestamptz(self, timestamptz_type: TimestamptzType) -> AvroType:
+        # Iceberg only supports micro's
+        return {"type": "long", "logicalType": "timestamp-micros", "adjust-to-utc": True}
+
+    def visit_string(self, string_type: StringType) -> AvroType:
+        return "string"
+
+    def visit_uuid(self, uuid_type: UUIDType) -> AvroType:
+        return {"type": "fixed", "size": 16, "logicalType": "uuid", "name": "uuid_fixed"}
+
+    def visit_binary(self, binary_type: BinaryType) -> AvroType:
+        return "bytes"
