@@ -15,15 +15,13 @@
 #  specific language governing permissions and limitations
 #  under the License.
 # pylint: disable=protected-access,redefined-outer-name
-import json
+import copy
 import uuid
-from typing import Any, Dict
-from unittest.mock import MagicMock, patch
+from unittest.mock import MagicMock, call, patch
 
 import pytest
-from hive_metastore.ttypes import AlreadyExistsException
-from hive_metastore.ttypes import Database as HiveDatabase
 from hive_metastore.ttypes import (
+    AlreadyExistsException,
     FieldSchema,
     InvalidOperationException,
     MetaException,
@@ -32,6 +30,7 @@ from hive_metastore.ttypes import (
     SkewedInfo,
     StorageDescriptor,
 )
+from hive_metastore.ttypes import Database as HiveDatabase
 from hive_metastore.ttypes import Table as HiveTable
 
 from pyiceberg.catalog import PropertiesUpdateSummary
@@ -42,10 +41,9 @@ from pyiceberg.exceptions import (
     NoSuchNamespaceError,
     NoSuchTableError,
 )
+from pyiceberg.partitioning import PartitionField, PartitionSpec
 from pyiceberg.schema import Schema
-from pyiceberg.serializers import ToOutputFile
-from pyiceberg.table.metadata import TableMetadataUtil, TableMetadataV2
-from pyiceberg.table.partitioning import PartitionField, PartitionSpec
+from pyiceberg.table.metadata import TableMetadataUtil, TableMetadataV1, TableMetadataV2
 from pyiceberg.table.refs import SnapshotRef, SnapshotRefType
 from pyiceberg.table.snapshots import (
     MetadataLogEntry,
@@ -61,6 +59,7 @@ from pyiceberg.table.sorting import (
     SortOrder,
 )
 from pyiceberg.transforms import BucketTransform, IdentityTransform
+from pyiceberg.typedef import UTF8
 from pyiceberg.types import (
     BooleanType,
     IntegerType,
@@ -68,18 +67,13 @@ from pyiceberg.types import (
     NestedField,
     StringType,
 )
-from tests.conftest import LocalFileIO
 
 HIVE_CATALOG_NAME = "hive"
 HIVE_METASTORE_FAKE_URL = "thrift://unknown:9083"
 
 
 @pytest.fixture
-def hive_table(tmp_path_factory, example_table_metadata_v2: Dict[str, Any]) -> HiveTable:
-    metadata_path = str(tmp_path_factory.mktemp("metadata") / f"{uuid.uuid4()}.metadata.json")
-    metadata = TableMetadataV2(**example_table_metadata_v2)
-    ToOutputFile.table_metadata(metadata, LocalFileIO().new_output(str(metadata_path)), True)
-
+def hive_table(metadata_location: str) -> HiveTable:
     return HiveTable(
         tableName="new_tabl2e",
         dbName="default",
@@ -118,7 +112,7 @@ def hive_table(tmp_path_factory, example_table_metadata_v2: Dict[str, Any]) -> H
             "EXTERNAL": "TRUE",
             "transient_lastDdlTime": "1659092339",
             "table_type": "ICEBERG",
-            "metadata_location": metadata_path,
+            "metadata_location": metadata_location,
         },
         viewOriginalText=None,
         viewExpandedText=None,
@@ -143,7 +137,7 @@ def hive_table(tmp_path_factory, example_table_metadata_v2: Dict[str, Any]) -> H
 
 
 @pytest.fixture(scope="session")
-def hive_database(tmp_path_factory) -> HiveDatabase:
+def hive_database(tmp_path_factory: pytest.TempPathFactory) -> HiveDatabase:
     # Pre-create the directory, this has to be done because
     # of a local FS. Not needed with an actual object store.
     database_path = tmp_path_factory.mktemp("database")
@@ -166,12 +160,12 @@ def hive_database(tmp_path_factory) -> HiveDatabase:
     )
 
 
-def test_no_uri_supplied():
+def test_no_uri_supplied() -> None:
     with pytest.raises(KeyError):
         HiveCatalog("production")
 
 
-def test_check_number_of_namespaces(table_schema_simple: Schema):
+def test_check_number_of_namespaces(table_schema_simple: Schema) -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     with pytest.raises(ValueError):
@@ -188,7 +182,7 @@ def test_check_number_of_namespaces(table_schema_simple: Schema):
 
 
 @patch("time.time", MagicMock(return_value=12345))
-def test_create_table(table_schema_simple: Schema, hive_database: HiveDatabase, hive_table: HiveTable):
+def test_create_table(table_schema_simple: Schema, hive_database: HiveDatabase, hive_table: HiveTable) -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -261,10 +255,10 @@ def test_create_table(table_schema_simple: Schema, hive_database: HiveDatabase,
         )
     )
 
-    with open(metadata_location, encoding="utf-8") as f:
-        payload = json.load(f)
+    with open(metadata_location, encoding=UTF8) as f:
+        payload = f.read()
 
-    metadata = TableMetadataUtil.parse_obj(payload)
+    metadata = TableMetadataUtil.parse_raw(payload)
 
     assert "database/table" in metadata.location
 
@@ -284,7 +278,7 @@ def test_create_table(table_schema_simple: Schema, hive_database: HiveDatabase,
         ],
         current_schema_id=0,
         last_partition_id=1000,
-        properties={"owner": "javaberg"},
+        properties={"owner": "javaberg", 'write.parquet.compression-codec': 'zstd'},
         partition_specs=[PartitionSpec()],
         default_spec_id=0,
         current_snapshot_id=None,
@@ -298,10 +292,63 @@ def test_create_table(table_schema_simple: Schema, hive_database: HiveDatabase,
         last_sequence_number=0,
     )
 
-    assert metadata.dict() == expected.dict()
+    assert metadata.model_dump() == expected.model_dump()
 
 
-def test_load_table(hive_table: HiveTable):
+@patch("time.time", MagicMock(return_value=12345))
+def test_create_v1_table(table_schema_simple: Schema, hive_database: HiveDatabase, hive_table: HiveTable) -> None:
+    catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
+
+    catalog._client = MagicMock()
+    catalog._client.__enter__().create_table.return_value = None
+    catalog._client.__enter__().get_table.return_value = hive_table
+    catalog._client.__enter__().get_database.return_value = hive_database
+    catalog.create_table(
+        ("default", "table"), schema=table_schema_simple, properties={"owner": "javaberg", "format-version": "1"}
+    )
+
+    # Test creating V1 table
+    called_v1_table: HiveTable = catalog._client.__enter__().create_table.call_args[0][0]
+    metadata_location = called_v1_table.parameters["metadata_location"]
+    with open(metadata_location, encoding=UTF8) as f:
+        payload = f.read()
+
+    expected_schema = Schema(
+        NestedField(field_id=1, name="foo", field_type=StringType(), required=False),
+        NestedField(field_id=2, name="bar", field_type=IntegerType(), required=True),
+        NestedField(field_id=3, name="baz", field_type=BooleanType(), required=False),
+        schema_id=0,
+        identifier_field_ids=[2],
+    )
+    actual_v1_metadata = TableMetadataUtil.parse_raw(payload)
+    expected_spec = PartitionSpec()
+    expected_v1_metadata = TableMetadataV1(
+        location=actual_v1_metadata.location,
+        table_uuid=actual_v1_metadata.table_uuid,
+        last_updated_ms=actual_v1_metadata.last_updated_ms,
+        last_column_id=3,
+        schema=expected_schema,
+        schemas=[expected_schema],
+        current_schema_id=0,
+        last_partition_id=1000,
+        properties={"owner": "javaberg", "write.parquet.compression-codec": "zstd"},
+        partition_spec=[],
+        partition_specs=[expected_spec],
+        default_spec_id=0,
+        current_snapshot_id=None,
+        snapshots=[],
+        snapshot_log=[],
+        metadata_log=[],
+        sort_orders=[SortOrder(order_id=0)],
+        default_sort_order_id=0,
+        refs={},
+        format_version=1,
+    )
+
+    assert actual_v1_metadata.model_dump() == expected_v1_metadata.model_dump()
+
+
+def test_load_table(hive_table: HiveTable) -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -358,8 +405,8 @@ def test_load_table(hive_table: HiveTable):
             ),
         ],
         snapshot_log=[
-            SnapshotLogEntry(snapshot_id="3051729675574597004", timestamp_ms=1515100955770),
-            SnapshotLogEntry(snapshot_id="3055729675574597004", timestamp_ms=1555100955770),
+            SnapshotLogEntry(snapshot_id=3051729675574597004, timestamp_ms=1515100955770),
+            SnapshotLogEntry(snapshot_id=3055729675574597004, timestamp_ms=1555100955770),
         ],
         metadata_log=[MetadataLogEntry(metadata_file="s3://bucket/.../v1.json", timestamp_ms=1515100)],
         sort_orders=[
@@ -397,11 +444,160 @@ def test_load_table(hive_table: HiveTable):
         last_sequence_number=34,
     )
 
-    assert table.identifier == ("default", "new_tabl2e")
+    assert table.identifier == (HIVE_CATALOG_NAME, "default", "new_tabl2e")
     assert expected == table.metadata
 
 
-def test_rename_table_from_does_not_exists():
+def test_load_table_from_self_identifier(hive_table: HiveTable) -> None:
+    catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
+
+    catalog._client = MagicMock()
+    catalog._client.__enter__().get_table.return_value = hive_table
+    intermediate = catalog.load_table(("default", "new_tabl2e"))
+    table = catalog.load_table(intermediate.identifier)
+
+    catalog._client.__enter__().get_table.assert_called_with(dbname="default", tbl_name="new_tabl2e")
+
+    expected = TableMetadataV2(
+        location="s3://bucket/test/location",
+        table_uuid=uuid.UUID("9c12d441-03fe-4693-9a96-a0705ddf69c1"),
+        last_updated_ms=1602638573590,
+        last_column_id=3,
+        schemas=[
+            Schema(
+                NestedField(field_id=1, name="x", field_type=LongType(), required=True),
+                schema_id=0,
+                identifier_field_ids=[],
+            ),
+            Schema(
+                NestedField(field_id=1, name="x", field_type=LongType(), required=True),
+                NestedField(field_id=2, name="y", field_type=LongType(), required=True, doc="comment"),
+                NestedField(field_id=3, name="z", field_type=LongType(), required=True),
+                schema_id=1,
+                identifier_field_ids=[1, 2],
+            ),
+        ],
+        current_schema_id=1,
+        partition_specs=[
+            PartitionSpec(PartitionField(source_id=1, field_id=1000, transform=IdentityTransform(), name="x"), spec_id=0)
+        ],
+        default_spec_id=0,
+        last_partition_id=1000,
+        properties={"read.split.target.size": "134217728"},
+        current_snapshot_id=3055729675574597004,
+        snapshots=[
+            Snapshot(
+                snapshot_id=3051729675574597004,
+                parent_snapshot_id=None,
+                sequence_number=0,
+                timestamp_ms=1515100955770,
+                manifest_list="s3://a/b/1.avro",
+                summary=Summary(operation=Operation.APPEND),
+                schema_id=None,
+            ),
+            Snapshot(
+                snapshot_id=3055729675574597004,
+                parent_snapshot_id=3051729675574597004,
+                sequence_number=1,
+                timestamp_ms=1555100955770,
+                manifest_list="s3://a/b/2.avro",
+                summary=Summary(operation=Operation.APPEND),
+                schema_id=1,
+            ),
+        ],
+        snapshot_log=[
+            SnapshotLogEntry(snapshot_id=3051729675574597004, timestamp_ms=1515100955770),
+            SnapshotLogEntry(snapshot_id=3055729675574597004, timestamp_ms=1555100955770),
+        ],
+        metadata_log=[MetadataLogEntry(metadata_file="s3://bucket/.../v1.json", timestamp_ms=1515100)],
+        sort_orders=[
+            SortOrder(
+                SortField(
+                    source_id=2, transform=IdentityTransform(), direction=SortDirection.ASC, null_order=NullOrder.NULLS_FIRST
+                ),
+                SortField(
+                    source_id=3,
+                    transform=BucketTransform(num_buckets=4),
+                    direction=SortDirection.DESC,
+                    null_order=NullOrder.NULLS_LAST,
+                ),
+                order_id=3,
+            )
+        ],
+        default_sort_order_id=3,
+        refs={
+            "test": SnapshotRef(
+                snapshot_id=3051729675574597004,
+                snapshot_ref_type=SnapshotRefType.TAG,
+                min_snapshots_to_keep=None,
+                max_snapshot_age_ms=None,
+                max_ref_age_ms=10000000,
+            ),
+            "main": SnapshotRef(
+                snapshot_id=3055729675574597004,
+                snapshot_ref_type=SnapshotRefType.BRANCH,
+                min_snapshots_to_keep=None,
+                max_snapshot_age_ms=None,
+                max_ref_age_ms=None,
+            ),
+        },
+        format_version=2,
+        last_sequence_number=34,
+    )
+
+    assert table.identifier == (HIVE_CATALOG_NAME, "default", "new_tabl2e")
+    assert expected == table.metadata
+
+
+def test_rename_table(hive_table: HiveTable) -> None:
+    catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
+
+    renamed_table = copy.deepcopy(hive_table)
+    renamed_table.dbName = "default"
+    renamed_table.tableName = "new_tabl3e"
+
+    catalog._client = MagicMock()
+    catalog._client.__enter__().get_table.side_effect = [hive_table, renamed_table]
+    catalog._client.__enter__().alter_table.return_value = None
+
+    from_identifier = ("default", "new_tabl2e")
+    to_identifier = ("default", "new_tabl3e")
+    table = catalog.rename_table(from_identifier, to_identifier)
+
+    assert table.identifier == ("hive",) + to_identifier
+
+    calls = [call(dbname="default", tbl_name="new_tabl2e"), call(dbname="default", tbl_name="new_tabl3e")]
+    catalog._client.__enter__().get_table.assert_has_calls(calls)
+    catalog._client.__enter__().alter_table.assert_called_with(dbname="default", tbl_name="new_tabl2e", new_tbl=renamed_table)
+
+
+def test_rename_table_from_self_identifier(hive_table: HiveTable) -> None:
+    catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
+
+    catalog._client = MagicMock()
+    catalog._client.__enter__().get_table.return_value = hive_table
+
+    from_identifier = ("default", "new_tabl2e")
+    from_table = catalog.load_table(from_identifier)
+    catalog._client.__enter__().get_table.assert_called_with(dbname="default", tbl_name="new_tabl2e")
+
+    renamed_table = copy.deepcopy(hive_table)
+    renamed_table.dbName = "default"
+    renamed_table.tableName = "new_tabl3e"
+
+    catalog._client.__enter__().get_table.side_effect = [hive_table, renamed_table]
+    catalog._client.__enter__().alter_table.return_value = None
+    to_identifier = ("default", "new_tabl3e")
+    table = catalog.rename_table(from_table.identifier, to_identifier)
+
+    assert table.identifier == ("hive",) + to_identifier
+
+    calls = [call(dbname="default", tbl_name="new_tabl2e"), call(dbname="default", tbl_name="new_tabl3e")]
+    catalog._client.__enter__().get_table.assert_has_calls(calls)
+    catalog._client.__enter__().alter_table.assert_called_with(dbname="default", tbl_name="new_tabl2e", new_tbl=renamed_table)
+
+
+def test_rename_table_from_does_not_exists() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -415,7 +611,7 @@ def test_rename_table_from_does_not_exists():
     assert "Table does not exist: does_not_exists" in str(exc_info.value)
 
 
-def test_rename_table_to_namespace_does_not_exists():
+def test_rename_table_to_namespace_does_not_exists() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -429,7 +625,7 @@ def test_rename_table_to_namespace_does_not_exists():
     assert "Database does not exists: default_does_not_exists" in str(exc_info.value)
 
 
-def test_drop_database_does_not_empty():
+def test_drop_database_does_not_empty() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -443,7 +639,7 @@ def test_drop_database_does_not_empty():
     assert "Database not_empty is not empty" in str(exc_info.value)
 
 
-def test_drop_database_does_not_exists():
+def test_drop_database_does_not_exists() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -455,7 +651,7 @@ def test_drop_database_does_not_exists():
     assert "Database does not exists: does_not_exists" in str(exc_info.value)
 
 
-def test_list_tables():
+def test_list_tables() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -474,7 +670,7 @@ def test_list_tables():
     catalog._client.__enter__().get_all_tables.assert_called_with(db_name="database")
 
 
-def test_list_namespaces():
+def test_list_namespaces() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -485,7 +681,7 @@ def test_list_namespaces():
     catalog._client.__enter__().get_all_databases.assert_called()
 
 
-def test_drop_table():
+def test_drop_table() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -496,7 +692,20 @@ def test_drop_table():
     catalog._client.__enter__().drop_table.assert_called_with(dbname="default", name="table", deleteData=False)
 
 
-def test_drop_table_does_not_exists():
+def test_drop_table_from_self_identifier(hive_table: HiveTable) -> None:
+    catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
+
+    catalog._client = MagicMock()
+    catalog._client.__enter__().get_table.return_value = hive_table
+    table = catalog.load_table(("default", "new_tabl2e"))
+
+    catalog._client.__enter__().get_all_databases.return_value = ["namespace1", "namespace2"]
+    catalog.drop_table(table.identifier)
+
+    catalog._client.__enter__().drop_table.assert_called_with(dbname="default", name="new_tabl2e", deleteData=False)
+
+
+def test_drop_table_does_not_exists() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -508,14 +717,14 @@ def test_drop_table_does_not_exists():
     assert "Table does not exists: does_not_exists" in str(exc_info.value)
 
 
-def test_purge_table():
+def test_purge_table() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     with pytest.raises(NotImplementedError):
         catalog.purge_table(("default", "does_not_exists"))
 
 
-def test_create_database():
+def test_create_database() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -542,7 +751,7 @@ def test_create_database():
     )
 
 
-def test_create_database_already_exists():
+def test_create_database_already_exists() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -554,7 +763,7 @@ def test_create_database_already_exists():
     assert "Database default already exists" in str(exc_info.value)
 
 
-def test_load_namespace_properties(hive_database: HiveDatabase):
+def test_load_namespace_properties(hive_database: HiveDatabase) -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -565,7 +774,7 @@ def test_load_namespace_properties(hive_database: HiveDatabase):
     catalog._client.__enter__().get_database.assert_called_with(name="default2")
 
 
-def test_load_namespace_properties_does_not_exists():
+def test_load_namespace_properties_does_not_exists() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -577,7 +786,7 @@ def test_load_namespace_properties_does_not_exists():
     assert "Database does not exists: does_not_exists" in str(exc_info.value)
 
 
-def test_update_namespace_properties(hive_database: HiveDatabase):
+def test_update_namespace_properties(hive_database: HiveDatabase) -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -608,7 +817,7 @@ def test_update_namespace_properties(hive_database: HiveDatabase):
     )
 
 
-def test_update_namespace_properties_namespace_does_not_exists():
+def test_update_namespace_properties_namespace_does_not_exists() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     catalog._client = MagicMock()
@@ -620,7 +829,7 @@ def test_update_namespace_properties_namespace_does_not_exists():
     assert "Database does not exists: does_not_exists" in str(exc_info.value)
 
 
-def test_update_namespace_properties_overlap():
+def test_update_namespace_properties_overlap() -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, uri=HIVE_METASTORE_FAKE_URL)
 
     with pytest.raises(ValueError) as exc_info:
@@ -629,7 +838,7 @@ def test_update_namespace_properties_overlap():
     assert "Updates and deletes have an overlap: {'a'}" in str(exc_info.value)
 
 
-def test_construct_hive_storage_descriptor_simple(table_schema_simple: Schema):
+def test_construct_hive_storage_descriptor_simple(table_schema_simple: Schema) -> None:
     descriptor = _construct_hive_storage_descriptor(table_schema_simple, "s3://")
     assert descriptor == StorageDescriptor(
         cols=[
@@ -659,7 +868,7 @@ def test_construct_hive_storage_descriptor_simple(table_schema_simple: Schema):
     )
 
 
-def test_construct_hive_storage_descriptor_nested(table_schema_nested: Schema):
+def test_construct_hive_storage_descriptor_nested(table_schema_nested: Schema) -> None:
     descriptor = _construct_hive_storage_descriptor(table_schema_nested, "s3://")
     assert descriptor == StorageDescriptor(
         cols=[
@@ -693,7 +902,7 @@ def test_construct_hive_storage_descriptor_nested(table_schema_nested: Schema):
     )
 
 
-def test_resolve_table_location_warehouse(hive_database: HiveDatabase):
+def test_resolve_table_location_warehouse(hive_database: HiveDatabase) -> None:
     catalog = HiveCatalog(HIVE_CATALOG_NAME, warehouse="/tmp/warehouse/", uri=HIVE_METASTORE_FAKE_URL)
 
     # Set this one to None, so we'll fall back to the properties
@@ -703,4 +912,4 @@ def test_resolve_table_location_warehouse(hive_database: HiveDatabase):
     catalog._client.__enter__().get_database.return_value = hive_database
 
     location = catalog._resolve_table_location(None, "database", "table")
-    assert location == "/tmp/warehouse/database/table"
+    assert location == "/tmp/warehouse/database.db/table"
